function [EST_SL1, EST_SL2, EST_L]=Est_Length(EST_C1, EST_C2)

% Computes the upstream extent of the saline intrusion - the estuarine
% length based on the depth and the M2 tidal amplitude at the mouth of the
% estuary (Prandle, 2004). The function uses the global bathymetry
% (SRTM15+) dataset (downloadable at:
% https://topex.ucsd.edu/WWW_html/srtm15_plus.html), and the global M2
% tidal amplitude (FES2004) dataset (downloadable at:
% http://www.legos.obs-mip.fr/en/share/soa/cgi/getarc/v0.0a/index.pl.cgi?contexte=SOA&donnees=maree&produit=modele_fes).
% These datasets are the highest resolution datasets available on a global
% base and can be applied for every estuary in the world. If local higher
% resolution datasets are desired, the datasets can be substituted. 
% Because bathymetry dataset may underestimate the depth of some estuaries,
% a minimum depth of 1.3 m (_fix) is applied based on the depth distribution of 62
% estuaries worldwide (Prandle, 2009; Savenije, 2005; Dronkers, 2017; Yates et al., 1996)
% 
% INPUTS :
%           EST_C1 - Matlab file containing centerline output of the
%           centerline_from_mask_MorphEst function (centerline) for time 1
%           EST_C2 - Matlab file containing centerline output of the
%           centerline_from_mask_MorphEst function (centelrine) for time 2
%
%
% OUTPUTS :
%           EST_SL1 - Matlab file containing information about the measured
%           estuarine length for time 1 in raster format
%           EST_SL2 - Matlab file containing information about the measured
%           estuarine length for time 2 in raster format
%           EST_L - Matlab file containing information about the estuarine
%           length in table format

[n,~]=size(EST_C2);

for i=1:n
%% Estuary Depth    
    disp('Estuary Depth')
    
    SRTM.Lon=ncread('SRTM15+V2.1.nc','lon');
    SRTM.Lat=ncread('SRTM15+V2.1.nc','lat');
    
    try
        if ~isempty(EST_C1(i).start_x) || ~isempty(EST_C2(i).start_x)
            start_x_coor=nanmean([EST_C1(i).start_x_coor,EST_C2(i).start_x_coor]);
            start_y_coor=nanmean([EST_C1(i).start_y_coor,EST_C2(i).start_y_coor]);
            
            a1=SRTM.Lon-start_x_coor; a11=abs(a1); [~,a12]=sort(a11); a13=sort(a12(1:50)); a14=a1(a13);
            a2=SRTM.Lat-start_y_coor; a21=abs(a2); [~,a22]=sort(a21); a23=sort(a22(1:50)); a24=a2(a23);
            [a15,a25]=meshgrid(a14,a24); a3=a15.^2+a25.^2;
            z=ncread('SRTM15+V2.1.nc','z',[min(a13),min(a23)],[length(a13),length(a23)]);
            z1=find(z<0); 
            if isempty(z1)
              EST_C1(i).M_depth=nan;
              EST_C2(i).M_depth=nan;
            else
                 z2=a3(z1); [~,z3]=sort(z2); z4=z3(1); z5=z1(z4);
                 EST_C1(i).M_depth=abs(z(z5));
                 EST_C2(i).M_depth=abs(z(z5));
            end
            
            if EST_C1(i).M_depth<1.3 || isnan(EST_C1(i).M_depth)
                EST_C1(i).M_depth_fix=1.3;
                EST_C2(i).M_depth_fix=1.3;
            else
                EST_C1(i).M_depth_fix=EST_C1(i).M_depth;
                EST_C2(i).M_depth_fix=EST_C2(i).M_depth;
            end
        end
        
    catch ME
        continue
    end

    clear a1 a11 a12 a13 a14 a15 a2 a21 a22 a23 a24 a25 a3 z z1 z2 z3 z4 z5
    disp('Estuary Depth : done')

%% Tidal amplitude
    disp('Tidal amplitude')
    
    T.Lon=ncread('tide.M2.nc','lon'); T.Lon=T.Lon(:,1);
    T.Lat=ncread('tide.M2.nc','lat'); T.Lat=T.Lat(1,:)';
    T.Ha=ncread('tide.M2.nc','Ha');

    try
        if ~isempty(EST_C1(i).start_x) || ~isempty(EST_C2(i).start_x)
             start_x_coor=nanmean([EST_C1(i).start_x_coor,EST_C2(i).start_x_coor]);
             start_y_coor=nanmean([EST_C1(i).start_y_coor,EST_C2(i).start_y_coor]);
             
             a1=T.Lon-start_x_coor; a11=abs(a1); [~,a12]=sort(a11); a13=sort(a12(1:200)); a14=a1(a13);
             a2=T.Lat-start_y_coor; a21=abs(a2); [~,a22]=sort(a21); a23=sort(a22(1:200)); a24=a2(a23);
             [a15,a25]=meshgrid(a14,a24); a3=a15.^2+a25.^2;
             t=T.Ha(a13,a23);
             t1=find(isnan(t)==0); t2=a3(t1); [~,t3]=sort(t2); t4=t3(1); t5=t1(t4);
             
             EST_C1(i).M_tamp=t(t5);
             EST_C2(i).M_tamp=t(t5);
             
             else
        end
         
    catch ME
        continue
    end
    
    try
        if isnan(EST_C1(i).M_tamp)
        a1=T.Lon-start_x_coor; a11=abs(a1); [~,a12]=sort(a11); a13=sort(a12(1:500)); a14=a1(a13);
        a2=T.Lat-start_y_coor; a21=abs(a2); [~,a22]=sort(a21); a23=sort(a22(1:500)); a24=a2(a23);
        [a15,a25]=meshgrid(a14,a24); a3=a15.^2+a25.^2;
        t=T.Ha(a13,a23);
        t1=find(isnan(t)==0); t2=a3(t1); [~,t3]=sort(t2); t4=t3(1); t5=t1(t4);
        
        EST_C1(i).M_tamp=t(t5);
        EST_C2(i).M_tamp=t(t5);
        
        else
        end
        
    catch ME
        continue
    end
    
    clear a1 a11 a12 a13 a14 a15 a2 a21 a22 a23 a24 a25 a3 t t1 t2 t3 t4 t5
    disp('Tidal amplitude : done')

%% Shorter one (1984 vs 2015)
    disp('Shorter one (1984 vs 2015)')

    try
    if ~isempty(EST_C1(i).start_x) || ~isempty(EST_C2(i).start_x)
    start_x_coor=nanmean([EST_C1(i).start_x_coor,EST_C2(i).start_x_coor]);
    start_y_coor=nanmean([EST_C1(i).start_y_coor,EST_C2(i).start_y_coor]);

    a=max(EST_C1(i).Length);
    b=max(EST_C2(i).Length);
    
    if a>b
        ind=find(EST_C1(i).Length>b);
        EST_C1(i).Rind(ind)=0;
    elseif a<b
        ind=find(EST_C2(i).Length>a);
        EST_C2(i).Rind(ind)=0;
    end
    
    else
    end
    
    catch ME
        continue
    end

    clear a b ind
    disp('Shorter one (1984 vs 2015) ; done')

%% Estuary length calculation
    disp('Estuary length calculation')

    try
    if ~isempty(EST_C1(i).start_x) || ~isempty(EST_C2(i).start_x)
        
    EST_C1(i).E_Length= 2460*((EST_C1(i).M_depth)^(5/4))/((EST_C1(i).M_tamp)^(1/2));
    EST_C2(i).E_Length= 2460*((EST_C2(i).M_depth)^(5/4))/((EST_C2(i).M_tamp)^(1/2));
    
    EST_C1(i).E_Length_fix= 2460*((EST_C1(i).M_depth_fix)^(5/4))/((EST_C1(i).M_tamp)^(1/2));
    EST_C2(i).E_Length_fix= 2460*((EST_C2(i).M_depth_fix)^(5/4))/((EST_C2(i).M_tamp)^(1/2));
    
    else
    end
    
    catch ME
        continue
    end

    disp('Estuary length calculation :done')

%% Save

    try
        if ~isempty(EST_C1(i).start_x) || ~isempty(EST_C2(i).start_x)
            
            start_x_coor=nanmean([EST_C1(i).start_x_coor,EST_C2(i).start_x_coor]);
            start_y_coor=nanmean([EST_C1(i).start_y_coor,EST_C2(i).start_y_coor]);
            
            EST_L(i,1).M_depth=EST_C1(i).M_depth;
            EST_L(i,1).M_depth_fix=EST_C1(i).M_depth_fix;
            EST_L(i,1).M_tamp=EST_C1(i).M_tamp;
            EST_L(i,1).E_Length=EST_C1(i).E_Length/1000;
            EST_L(i,1).E_Length_fix=EST_C1(i).E_Length_fix/1000;
            EST_L(i,1).start_x_coor=start_x_coor;
            EST_L(i,1).start_y_coor=start_y_coor;
        else
        end
        
    catch ME
        continue
    end
    
    disp({i,'/',n})    
    
end

%% Save

    EST_SL1=EST_C1;
    EST_SL2=EST_C2;
    
    disp('Est_Length : done')
end