function [EST_C]=banklines_from_mask_MorphEst(EST_C)

% Modified from RivMAP's banklines_from_mask function (Schwenk et al.,
% 2016)
% 
% Copyright (c) 2017, Jon
% Copyright (c) 2005-prese, Jos van der Geest
% Copyright (c) 2017, Douglas M. Schwarz
% Copyright (c) 2012, John D'Errico
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
% 
% * Redistributions of source code must retain the above copyright notice, this
%   list of conditions and the following disclaimer.
% 
% * Redistributions in binary form must reproduce the above copyright notice,
%   this list of conditions and the following disclaimer in the documentation
%   and/or other materials provided with the distribution
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
% FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
% DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
% CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
% OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%
%
% Computes bank lines from a binary channel mask.
%
% Requires the Image Processing toolbox
%
% The modified version now supports branching channels. Here, buffers are
% drawn around the centerline points with a radius defined as the the width
% at the mouth (assuming that the mouth width is the widest point of the
% estuary). Channel pixels outside of the buffer are then deleted.
%
% INPUTS :
%           EST_C - Matlab file containing output of the
%           centerline_from_mask_MorphEst tool
%
%
% OUTPUTS :
%           EST_C - Matlab file containing information about bank lines of the main estuary channel.
%           2x1 cell; first element is left bank, secon dis right bank.
%           Each cell contains a 2-column array of x and y coordinates. x,
%           y coordinates are returned with repsect to Matlab's image
%           plotting coordinates; namely, the origin as the top-left of the
%           image, not the bottom left. (Must multiply y-axis by -1 for
%           'regular' plotting unless plotting on top of an image).
%           Coordinates are returned in an upstrea-to-downstream
%           orientation. 

[n,~]=size(EST_C);

for i=1:n
    %% Buffer
    disp('Buffer')

    a=ceil(EST_C(i).rad);
    k=1:2*a+1;
    l=1:2*a+1;
    [c,d]=meshgrid(k,l);
    A=zeros(EST_C(i).row,EST_C(i).col);
    leng=sqrt((c-(a+1)).^2+(d-(a+1)).^2);
    ind=find(leng>a);
    c(ind)=nan; c=c-(a+1);
    d(ind)=nan; d=d-(a+1);
    
    for j=1:length(EST_C(i).cl)
        c1=c+EST_C(i).cl(j,1);
        d1=d+EST_C(i).cl(j,2);
        ind=find(d1>0 & d1<=EST_C(i).row & c1>0 & c1<=EST_C(i).col);
        d2=d1(ind); c2=c1(ind);
        for k=1:length(ind)
            A(d2(k),c2(k))=A(d2(k),c2(k))+1;
        end
        if mod(j,100)==1
            disp({j,'/',num2str(length(EST_C(i).cl))})
        end
        clear c1 c2 d1 d2
    end
    
    EST_C(i).data1=EST_C(i).data;
    EST_C(i).data1(A==0)=0;
    
    clear j k l a b c d leng ind A
    disp('Buffer : done')
    
%% Smoothing
    disp('Smoothing + adjoint cut point')
    
    EST_C(i).data2 = imfill(EST_C(i).data1,8,'holes');
    Aa=imdilate(EST_C(i).Icl,strel('rectangle',[3,3]));
    A=imdilate(EST_C(i).Icl,strel('disk',3));
    A(3:end-2,3:end-2)=Aa(3:end-2,3:end-2);
    ind=find(A==1 & EST_C(i).data2==0);
    EST_C(i).data3=EST_C(i).data2;
    EST_C(i).data3(ind) = 1;
    
    clear Aa
    disp('Smoothing + adjoint cut point : done')

%% Add corner
    disp('Add corner')

    A=zeros(EST_C(i).row+4,EST_C(i).col+4);
    A(3:EST_C(i).row+2,3:EST_C(i).col+2)=EST_C(i).data3;
    B=EST_C(i).cl([1,end],:);
    for j=1:2
        if strcmp(EST_C(i).dir(j),'N')
            b=find(EST_C(i).data3(1,:)==1);
            b1=b-B(j,1); b2=find(abs(b1)==min(abs(b1)),1); b3=b1-b1(b2); b4=[1:length(b1)]'-b2; b5=b3(:)-b4(:); b6=find(b5==0);
            A(1:2,min(b(b6))+1:max(b(b6))+3)=1;
            a=B(j,1)-1:B(j,1)+1;
            if min(a)<=1
               A(1:2,a+(1-min(a)))=1;
            elseif max(a)>=EST_C(i).col+1
                A(1:2,a+2-(EST_C(i).col-max(a)))=1;
            else, A(1:2,a)=1;
            end            
        elseif strcmp(EST_C(i).dir(j),'E')
            b=find(EST_C(i).data3(:,EST_C(i).col)==1);
            b1=b-B(j,2); b2=find(abs(b1)==min(abs(b1)),1); b3=b1-b1(b2); b4=[1:length(b1)]'-b2; b5=b3(:)-b4(:); b6=find(b5==0);
            A(min(b(b6))+2:max(b(b6))+2,end-1:end)=1;
            a=B(j,2)-1:B(j,2)+1;
            if min(a)<=1
                A(a+(1-min(a)),end-1:end)=1;
            elseif max(a)>=EST_C(i).row+1
                A(a+2-(EST_C(i).row-max(a)),end-1:end)=1;
            else, A(a,end-1:end)=1;
            end
        elseif strcmp(EST_C(i).dir(j),'W')
            b=find(EST_C(i).data3(:,1)==1);
            b1=b-B(j,2); b2=find(abs(b1)==min(abs(b1)),1); b3=b1-b1(b2); b4=[1:length(b1)]'-b2; b5=b3(:)-b4(:); b6=find(b5==0);
            A(min(b(b6))+2:max(b(b6))+2,1:2)=1;
            a=B(j,2)-1:B(j,2)+1;
            if min(a)<=1
                A(a+(1-min(a)),1:2)=1;
            elseif max(a)>=EST_C(i).row+1
                A(a+2-(EST_C(i).row-max(a)),1:2)=1;
            else, A(a,1:2)=1;
            end
        elseif strcmp(EST_C(i).dir(j),'S')
            b=find(EST_C(i).data3(EST_C(i).row,:)==1);
            b1=b-B(j,1); b2=find(abs(b1)==min(abs(b1)),1); b3=b1-b1(b2); b4=[1:length(b1)]'-b2; b5=b3(:)-b4(:); b6=find(b5==0);
            A(end-1:end,min(b(b6))+2:max(b(b6))+2)=1;
            a=B(j,1)-1:B(j,1)+1;
            if min(a)<=1
                A(end-1:end,a+(1-min(a)))=1;
            elseif max(a)>=EST_C(i).col+1
                A(end-1:end,a+2-(EST_C(i).col-max(a)))=1;
            else, A(end-1:end,a)=1;
            end
        end
    end
    
    clear j
    disp('Add corner : done')
    
%% Add corner for banks
    disp('Add corner for banks')
    
    B=imdilate(A,strel('disk',1));
    A(3:end-2,3:end-2)=B(3:end-2,3:end-2);
    A([1,2],:)=0; A([end-1,end],:)=0; A(:,[1,2])=0; A(:,[end-1,end])=0;
    B=EST_C(i).cl([1,end],:);
    for j=1:2
        if strcmp(EST_C(i).dir(j),'N')
            b=find(EST_C(i).data3(1,:)==1);
            b1=b-B(j,1); b2=find(abs(b1)==min(abs(b1)),1); b3=b1-b1(b2); b4=[1:length(b1)]'-b2; b5=b3(:)-b4(:); b6=find(b5==0);
            A(1:2,min(b(b6))+1:max(b(b6))+3)=1;
            a=B(j,1)-1:B(j,1)+1;
            if min(a)<=1
               A(1:2,a+(1-min(a)))=1;
            elseif max(a)>=EST_C(i).col+1
                A(1:2,a+2-(EST_C(i).col-max(a)))=1;
            else, A(1:2,a)=1;
            end            
        elseif strcmp(EST_C(i).dir(j),'E')
            b=find(EST_C(i).data3(:,EST_C(i).col)==1);
            b1=b-B(j,2); b2=find(abs(b1)==min(abs(b1)),1); b3=b1-b1(b2); b4=[1:length(b1)]'-b2; b5=b3(:)-b4(:); b6=find(b5==0);
            A(min(b(b6))+2:max(b(b6))+2,end-1:end)=1;
            a=B(j,2)-1:B(j,2)+1;
            if min(a)<=1
                A(a+(1-min(a)),end-1:end)=1;
            elseif max(a)>=EST_C(i).row+1
                A(a+2-(EST_C(i).row-max(a)),end-1:end)=1;
            else, A(a,end-1:end)=1;
            end
        elseif strcmp(EST_C(i).dir(j),'W')
            b=find(EST_C(i).data3(:,1)==1);
            b1=b-B(j,2); b2=find(abs(b1)==min(abs(b1)),1); b3=b1-b1(b2); b4=[1:length(b1)]'-b2; b5=b3(:)-b4(:); b6=find(b5==0);
            A(min(b(b6))+2:max(b(b6))+2,1:2)=1;
            a=B(j,2)-1:B(j,2)+1;
            if min(a)<=1
                A(a+(1-min(a)),1:2)=1;
            elseif max(a)>=EST_C(i).row+1
                A(a+2-(EST_C(i).row-max(a)),1:2)=1;
            else, A(a,1:2)=1;
            end
        elseif strcmp(EST_C(i).dir(j),'S')
            b=find(EST_C(i).data3(EST_C(i).row,:)==1);
            b1=b-B(j,1); b2=find(abs(b1)==min(abs(b1)),1); b3=b1-b1(b2); b4=[1:length(b1)]'-b2; b5=b3(:)-b4(:); b6=find(b5==0);
            A(end-1:end,min(b(b6))+2:max(b(b6))+2)=1;
            a=B(j,1)-1:B(j,1)+1;
            if min(a)<=1
                A(end-1:end,a+(1-min(a)))=1;
            elseif max(a)>=EST_C(i).col+1
                A(end-1:end,a+2-(EST_C(i).col-max(a)))=1;
            else, A(end-1:end,a)=1;
            end
        end
    end
    A(1,1)=0; A(end,end)=0; A(end,1)=0; A(1,end)=0;
    EST_C(i).data4=A;
    EST_C(i).data4=EST_C(i).data4(3:end-2,3:end-2);
    EST_C(i).A=A;
    
    clear B
    disp('Add corner for banks : done')
    
%% Banks
    
    %%
    I=EST_C(i).A;
    es=EST_C(i).dir;
    
    if sum(sum(I)) < 100
        disp('No mask detected in input image')
        banksout{1} = [];
        banksout{2} = [];
        return
    end

    % Fill any holes
    I = imfill(I,'holes');
    
    % Remove any spurious small patches by keeping only largest blob
    sp = regionprops(I,'Area','PixelIdxList');
    [~,maxidx] = max([sp(:).Area]);
    I = false(size(I));
    I(sp(maxidx).PixelIdxList) = true;
    
    % Crop mask to edges of image
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    Icrop=I;
    % Original image size
    sizeIo = size(Icrop);

    % Find x,y coordinates of mask boundaries
    boundaries = cell2mat(bwboundaries(Icrop));
    x = boundaries(:,2);  
    y = boundaries(:,1);  

    % Perform cropping; order of cropping is important (SE, then NW)
    if ismember('S',es) == 1
        yBottom = ceil(max(y));
        Icrop(yBottom+1:end,:)=[];
    else
        yBottom = sizeIo(1);
    end
    if ismember('E',es) == 1
        xRight = ceil(max(x));
        Icrop(:,xRight+1:end) = [];
    else
        xRight = sizeIo(2);
    end
    if ismember('N',es) == 1
        yTop = ceil(min(y));
        Icrop(1:yTop-1,:)=[];
    else
        yTop = 1;
    end
    if ismember('W',es) == 1
        xLeft = ceil(min(x));
        Icrop(:,1:xLeft-1) = [];
    else
        xLeft = 1;
    end

    % Fill in any holes in the edges of the binary mask
    % Bottom
    Icrop(end+1,:) = true;
    Icrop = imfill(Icrop,'holes');
    % Top
    Icrop(2:end,:) = Icrop(1:end-1,:);
    Icrop(1,:) = true;
    Icrop = imfill(Icrop,'holes');
    Icrop(1,:) = [];
    % Right
    Icrop(:,end+1) = true;
    Icrop = imfill(Icrop,'holes');
    % Left
    Icrop(:,2:end) = Icrop(:,1:end-1);
    Icrop(:,1) = true;
    Icrop = imfill(Icrop,'holes');
    Icrop(:,1) = [];
    Icropped=Icrop;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % Find perimeter of mask
    Iperim = bwperim(Icropped);
    Ibanks = Iperim;
    
    % Remove unwanted boundary pixels by removing endpoints
    E = find(bwmorph(Ibanks,'endpoints'));
    count = 0;
    while numel(E)>0
        Ibanks(E) = false;
        E = find(bwmorph(Ibanks,'endpoints'));
        count = count + 1;
        if count > 25 % Try removing extra pixels with spur, skel commands
            Ibanks = bwmorph(Ibanks,'skel','Inf');
            Ibanks = bwmorph(Ibanks,'spur',1);
            break
        end
    end

    % Erase boundaries that are not part of banklines
    % Lots of code required for handling a few boundary pixels
    for k = 1:2
        if strcmp(es(k),'E') == 1
            % Remove border pixels
            Ibanks(:,end) = false;
            % Remove unattached groups
            Ibanks = bwareaopen(Ibanks,20);
            % Add proper border pixels back to perimeter
            [yp, xp] = find(Ibanks);
            x = size(Ibanks,2)-1;
            edgeminusonevals = yp(xp==x);
            diffs = diff(edgeminusonevals);
            [~,maxidx] = max(abs(diffs));
            miny = edgeminusonevals(maxidx);
            maxy =(edgeminusonevals(maxidx+1));
            for mm = 1:2
                if mm == 1
                    y = maxy;
                    if Iperim(y+1,x+1) == 1
                        Ibanks(y+1,x+1) = 1;
                    elseif Iperim(y,x+1) == 1
                        Ibanks(y,x+1) = 1;
                    elseif Iperim(y-1,x+1) == 1
                        Ibanks(y-1,x+1) = 1;
                    end
                else
                    y = miny;
                    if Iperim(y-1,x+1) == 1
                        Ibanks(y-1,x+1) = 1;
                    elseif Iperim(y,x+1) == 1
                        Ibanks(y,x+1) = 1;
                    elseif Iperim(y+1,x+1) == 1
                        Ibanks(y+1,x+1) = 1;
                    end
                end
            end
        elseif strcmp(es(k),'W') == 1
            % Remove border pixels
            Ibanks(:,1) = false;
            % Remove unattached groups
            Ibanks = bwareaopen(Ibanks,20);
            % Add proper border pixels back to perimeter
            [yp, xp] = find(Ibanks);
            x = 2;
            edgeminusonevals = yp(xp==x);
            diffs = diff(edgeminusonevals);
            [~,maxidx] = max(abs(diffs));
            miny = edgeminusonevals(maxidx);
            maxy =(edgeminusonevals(maxidx+1));
            for mm = 1:2
                if mm == 1
                    y = maxy;
                    if Iperim(y+1,x-1) == 1
                        Ibanks(y+1,x-1) = 1;
                    elseif Iperim(y,x-1) == 1
                        Ibanks(y,x-1) = 1;
                    elseif Iperim(y-1,x-1) == 1
                        Ibanks(y-1,x-1) = 1;
                    end
                else
                    y = miny;
                    if Iperim(y-1,x-1) == 1
                        Ibanks(y-1,x-1) = 1;
                    elseif Iperim(y,x-1) == 1
                        Ibanks(y,x-1) = 1;
                    elseif Iperim(y+1,x-1) == 1
                        Ibanks(y+1,x-1) = 1;
                    end
                end
            end
        elseif strcmp(es(k),'N') == 1
            % Remove border pixels
            Ibanks(1,:) = false;
            % Remove unattached groups
            Ibanks = bwareaopen(Ibanks,20);
            % Add proper border pixels back to perimeter
            [yp, xp] = find(Ibanks);
            y = 2;
            edgeminusonevals = xp(yp==y);
            diffs = diff(edgeminusonevals);
            [~,maxidx] = max(abs(diffs));
            minx = edgeminusonevals(maxidx);
            maxx = edgeminusonevals(maxidx+1);
            for mm = 1:2
                if mm == 1
                    x = maxx;
                    if Iperim(y-1,x+1) == 1
                        Ibanks(y-1,x+1) = 1;
                    elseif Iperim(y-1,x) == 1
                        Ibanks(y-1,x) = 1;
                    elseif Iperim(y-1,x-1) == 1
                        Ibanks(y-1,x-1) = 1;
                    end
                else
                    x = minx;
                    if Iperim(y-1,x-1) == 1
                        Ibanks(y-1,x-1) = 1;
                    elseif Iperim(y-1,x) == 1
                        Ibanks(y-1,x) = 1;
                    elseif Iperim(y-1,x+1) == 1
                        Ibanks(y-1,x+1) = 1;
                    end
                end
            end
        elseif strcmp(es(k),'S') == 1
            % Remove border pixels
            Ibanks(end,:) = false;
            % Remove unattached groups
            Ibanks = bwareaopen(Ibanks,20);
            % Add proper border pixels back to perimeter
            [yp, xp] = find(Ibanks);
            y = size(Ibanks,1)-1;        
            edgeminusonevals = xp(yp==y);
            diffs = diff(edgeminusonevals);
            [~,maxidx] = max(abs(diffs));
            minx = edgeminusonevals(maxidx);
            maxx = edgeminusonevals(maxidx+1);        
           for mm = 1:2
                if mm == 1
                    x = maxx;
                    if Iperim(y+1,x+1) == 1
                        Ibanks(y+1,x+1) = 1;
                    elseif Iperim(y+1,x) == 1
                        Ibanks(y+1,x) = 1;
                    elseif Iperim(y+1,x-1) == 1
                        Ibanks(y+1,x-1) = 1;
                    end
                else
                    x = minx;
                    if Iperim(y+1,x-1) == 1
                        Ibanks(y+1,x-1) = 1;
                    elseif Iperim(y+1,x) == 1
                        Ibanks(y+1,x) = 1;
                    elseif Iperim(y+1,x+1) == 1
                        Ibanks(y+1,x+1) = 1;
                    end
                end
            end
        end
    end
    
    % Extract each bank 
    ccbanks = bwconncomp(Ibanks);
    % parfor lr = 1:2
    if size(ccbanks.PixelIdxList,2)>2
        for k=1:size(ccbanks.PixelIdxList,2)
            ccbanks.Pix(k)=length(ccbanks.PixelIdxList{k});
        end
        [~,ind2]=sort(ccbanks.Pix,'descend')
        ind=ind2(1:2);
        ind=sort(ind);
    else, ind=[1,2];
    end
    
    for lr = 1:2
        % Make banks into image
        Itemp = false(size(Ibanks));
        Itemp(ccbanks.PixelIdxList{ind(lr)}) = true;
          
        % Find "true" endpoints of centerline
        [ptsy,ptsx] = find(Itemp);
        E = [find(ptsy==1); find(ptsy==size(Itemp,1)); find(ptsx==1); find(ptsx==size(Itemp,2))];
        Eind = sub2ind(size(Itemp),ptsy(E),ptsx(E));
               
        if numel(Eind) ~= 2
            disp(['Not two endpoints, lr=',num2str(lr)])
        end
    
        % Find shortest path between endpoints (cuts off spurious loops along bank edge)
        D1 = bwdistgeodesic(Itemp,Eind(1));
        D2 = bwdistgeodesic(Itemp,Eind(2));
        Dadd = D1 + D2;
        Dadd = round(Dadd * 8) / 8;
        Dadd(isnan(Dadd)) = inf;
        Itemp = imregionalmin(Dadd);
        Itemp = bwmorph(Itemp,'thin'); 
            
        % Trace skeleton to find bankline coordinates
        %%%%%%%%%%%%%%%%%%%%%%
        % Find upstream->downstream ordered centerline coordinates
        Iskel=Itemp;
        xst=ptsx(E(1));
        yst=ptsy(E(1));

        x = xst;
        y = yst;
        c = 1; 
        [nr, nc] = size(Iskel);

        while 1
            if y(c)-1 > 0 && Iskel(y(c)-1,x(c)) == 1  % N
                x(c+1) = x(c);
                y(c+1) = y(c) - 1;
                Iskel(y(c),x(c)) = 0;
            elseif x(c) + 1 <= nc && Iskel(y(c),x(c)+1) == 1  % E
                x(c+1) = x(c) + 1;
                y(c+1) = y(c);
                Iskel(y(c),x(c)) = 0;
            elseif y(c) + 1 <= nr && Iskel(y(c)+1,x(c)) == 1  % S
                x(c+1) = x(c);
                y(c+1) = y(c) + 1;
                Iskel(y(c),x(c)) = 0;
            elseif x(c) - 1 > 0 && Iskel(y(c),x(c)-1) == 1  % W
                x(c+1) = x(c) - 1;
                y(c+1) = y(c);
                Iskel(y(c),x(c)) = 0;
            elseif y(c)-1 > 0 && x(c) + 1 <= nc && Iskel(y(c)-1,x(c)+1) == 1  % NE
                x(c+1) = x(c) + 1;
                y(c+1) = y(c) - 1;
                Iskel(y(c),x(c)) = 0;
            elseif y(c) + 1 <= nr && x(c) + 1 <= nc && Iskel(y(c)+1,x(c)+1) == 1  % SE
                x(c+1) = x(c) + 1;
                y(c+1) = y(c) + 1;
                Iskel(y(c),x(c)) = 0;
            elseif y(c) + 1 <= nr && x(c) - 1 > 0 && Iskel(y(c)+1,x(c)-1) == 1  % SW
                x(c+1) = x(c) - 1;
                y(c+1) = y(c) + 1;
                Iskel(y(c),x(c)) = 0;
            elseif y(c)-1 > 0 && x(c) - 1 > 0 && Iskel(y(c)-1,x(c)-1) == 1  % NW
                x(c+1) = x(c) - 1;
                y(c+1) = y(c) - 1;
                Iskel(y(c),x(c)) = 0;
            elseif c > length(Iskel)*3
                break
            else
                break
            end
            c = c + 1;
        end 
        xy = [x; y]';

        % Order coordinates upstream->downstream
        if es(1) == 'W'
            if xy(1,1) > xy(end,1)
                xy = flipud(xy);
            end
        elseif es(1) == 'N'
            if xy(1,2) > xy(end,2)
                xy = flipud(xy);
            end
        elseif es(1) == 'E'
            if xy(1,1) < xy(end,1)
                xy = flipud(xy);
            end
        elseif es(1) == 'S'
            if xy(1,2) < xy(end,2)
                xy = flipud(xy);
            end
        end
        %%%%%%%%%%%%%%%%%%%

        % Adjust coordinates to original reference frame
        xy(:,1) = xy(:,1) + xLeft - 1;
        xy(:,2) = xy(:,2) + yTop - 1;
      
        banks{lr} = xy;
    end

    % Find which bank is left and which is right
    banksout{2} = [];
    if es(1) == 'S'
        if banks{1}(1,1) < banks{2}(1,1)
            banksout{1} = banks{1};
            banksout{2} = banks{2};
        else
            banksout{1} = banks{2};
            banksout{2} = banks{1};
        end
    elseif es(1) == 'N'
        if banks{1}(1,1) > banks{2}(1,1)
            banksout{1} = banks{1};
            banksout{2} = banks{2};
        else
            banksout{1} = banks{2};
            banksout{2} = banks{1};
        end
    elseif es(1) == 'E'
        if banks{1}(1,2) < banks{2}(1,2)
            banksout{1} = banks{1};
            banksout{2} = banks{2};
        else
            banksout{1} = banks{2};
            banksout{2} = banks{1};
        end
    elseif es(1) == 'W'
        if banks{1}(1,2) > banks{2}(1,2)
            banksout{1} = banks{1};
            banksout{2} = banks{2};
        else
            banksout{1} = banks{2};
            banksout{2} = banks{1};
        end
    end    
    
    EST_C(i).bank=banksout;
    
    %%
    EST_C(i).bank{1,1}=EST_C(i).bank{1,1}-2; EST_C(i).bank{1,2}=EST_C(i).bank{1,2}-2;
    ind11=find(EST_C(i).bank{1}(:,1)<1); ind12=find(EST_C(i).bank{1}(:,2)<1);
    ind21=find(EST_C(i).bank{2}(:,1)<1); ind22=find(EST_C(i).bank{2}(:,2)<1);
    ind1=[ind11;ind12]; ind1=unique(ind1); ind2=[ind21;ind22]; ind2=unique(ind2);
    EST_C(i).bank{1}(ind1,:)=[]; EST_C(i).bank{2}(ind2,:)=[];

%% Save

    EST_C=EST_C;
    
    disp({i,'/',n})    
    
end
    disp('banklines_from_mask_MorphEst : done')
end