function [EST_C]=centerline_from_mask_MorphEst(A1,A2,A3)

% Modified from RivMAP's centerline_from_mask function (Schwenk et al.,
% 2016) (% https://de.mathworks.com/matlabcentral/fileexchange/58264-rivmap-river-morphodynamics-from-analysis-of-planforms)
% 
% Copyright (c) 2017, Jon
% Copyright (c) 2005-prese, Jos van der Geest
% Copyright (c) 2017, Douglas M. Schwarz
% Copyright (c) 2012, John D'Errico
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
% 
% * Redistributions of source code must retain the above copyright notice, this
%   list of conditions and the following disclaimer.
% 
% * Redistributions in binary form must reproduce the above copyright notice,
%   this list of conditions and the following disclaimer in the documentation
%   and/or other materials provided with the distribution
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
% FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
% DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
% CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
% OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

% Computes a centerline from an input binary estuary channel mask and the
% coordinates and width of the estuary mouth. If the mask is a
% multi-threaded channel, the 'holes' between channel threads are filled
% and the returned centerline will be relative to the overall channel
% extent rather than individual threads. Spurious branches along the
% centerline are removed so that the final centerline follows the overall
% channel structure.
%
% The modified version automatically determines the two-element string specifying the sides of the image 
% that the channel intersects (e.g., 'SW', 'NS', etc.) based on the location of the estuary mouth.
% The first character is the upstream intersection.
%
% Requires the Image Processing toolbox
%
% INPUTS :
%           A1 - directory of text files with x and y coordinates
%           in meters as well as the width of the estuary mouth in meters
%           ex) for Han River: 14100312.67730, 4547551.73300, 2028.00000
%           ex) A1=dir('D:\Algorithm\*_coords1.txt');
%
%           A2 - directory of estuary binary text files (channel masks)with
%           information about the x and y coordinates of the lower left
%           corner of the channel mask raster in meters
%           ex) for Han River: 
%           ncols         4970
%           nrows         5659
%           xllcorner     14098971.658099
%           yllcorner     4427064.1414765
%           cellsize      30
%           NODATA_value  -9999
%           This is the standard format of estuary binary text files
%           converted from ArcGIS raster files (Raster to ASCII tool)
%           ex) A2=dir('D:\Algorithm\*_r84.txt');
%
%           A3 - directory of text files with x and y coordinates
%           in decimal degrees as well as the width of the estuary mouth in meters
%           ex) for Han River: 126.66526, 37.77402, 2028.00000
%           ex) A3=dir('D:\Algorithm\*proj_coords.txt');
%
%
% OUTPUTS :
%           EST_C - Matlab file containing information about Nx2 vector of
%           channel coordinates arranged in upstream->downstream order (cl) as
%           well as an image of centerline pixels (Icl)          

if nargin ~= 3
    disp('Not enough inputs')
    return
end

[n,~]=size(A1);

for i=1:n
    %% Extracting data
    disp('Extracting data')
    
    fileID1 = fopen(A1(i).name); fileID2 = fopen(A2(i).name); fileID3 = fopen(A3(i).name);
    if fileID1 < 0 || fileID2 < 0 || fileID3 < 0
        disp('text files directory is not registered')
        return
    end
    
    A4 = textscan(fileID2,'%s %s',6,'Delimiter',{'/t',' '},'MultipleDelimsAsOne',true);
    Est.col=str2double(A4{1,2}{1,1});
    Est.row=str2double(A4{1,2}{2,1});
    Est.x_con=str2double(A4{1,2}{3,1});
    Est.y_con=str2double(A4{1,2}{4,1});
    Est.cell=str2double(A4{1,2}{5,1});
    
    A5 = textscan(fileID2,'%f','Delimiter',{'/t'});
    Est.data=A5{1,1};
    
    A6 = textscan(fileID1,'%20.20f %20.20f %20.20f %20.20f','Delimiter',{','});
    Est.x_start=A6{1,1};
    Est.y_start=A6{1,2};
    Est.M_width=A6{1,3};
    Est.E_length=A6{1,4};
    
    A7 = textscan(fileID3,'%20.20f %20.20f %20.20f %20.20f','Delimiter',{','});
    Est.x_start_coor=A7{1,1};
    Est.y_start_coor=A7{1,2};

    fclose(fileID1); fclose(fileID2); fclose(fileID3);
    
    Est.data=reshape(Est.data,Est.col,Est.row);
    Est.data=Est.data';
    Est.data(Est.data<0)=0;
    Est.Ex_r=0;
    Est.Ex_d=0;
    
    Est.data(Est.data~=0)=1;
    
    disp('Extracting data : done')
    
%% finding start row/col from start point 
    disp('finding start row/col from start point')

    a=Est.x_start-Est.x_con;
    b=Est.y_start-Est.y_con;
    
    Est.x_point=round(a/Est.cell);
    Est.y_point=Est.row-round(b/Est.cell);
    
    [Est.row,Est.col]=size(Est.data);
    
    disp('finding start row/col from start point : done')
    
%% cutting blank edge
    disp('cutting blank edge')
    
    for j=1:Est.row
        c(j)=sum(Est.data(j,:)==1);
    end
    for j=1:Est.col
        d(j)=sum(Est.data(:,j)==1);
    end
    c=find(c==0); c1=c(c>Est.y_point); c2=c(c<Est.y_point);
    d=find(d==0); d1=d(d>Est.x_point); d2=d(d<Est.x_point);
    
    Est.x_point=Est.x_point-length(d2);
    Est.y_point=Est.y_point-length(c2);
    Est.col=Est.col-length(d1)-length(d2);
    Est.row=Est.row-length(c1)-length(c2);
    Est.data(c,:)=[];
    Est.data(:,d)=[];
    
    disp('cutting blank edge : done')
    
%% start point direction?
    disp('start point direction')

    if Est.y_point>Est.row/Est.col*Est.x_point && Est.y_point+Est.row/Est.col*Est.x_point>Est.row
        Est.S_dir='S';
    elseif Est.y_point>=Est.row/Est.col*Est.x_point && Est.y_point+Est.row/Est.col*Est.x_point<=Est.row
       Est.S_dir='W';
    elseif Est.y_point<=Est.row/Est.col*Est.x_point && Est.y_point+Est.row/Est.col*Est.x_point>Est.row
       Est.S_dir='E';
    elseif Est.y_point<=Est.row/Est.col*Est.x_point &&Est.y_point+Est.row/Est.col*Est.x_point<Est.row
        Est.S_dir='N';
    end

    disp('start point direction : done')
    
%% finding possibilities
    disp('finding possibilities')

    dir={'N';'E';'W';'S'};
    if ~isempty(Est.data)
    a=strfind(dir,Est.S_dir);
    b(1,1)=isempty(a{1,1}); b(2,1)=isempty(a{2,1}); b(3,1)=isempty(a{3,1}); b(4,1)=isempty(a{4,1});
    b=find(b==1);
    for j=1:3
       dirr{j,1}=dir{b(j),1};
       dirr{j,1}=strcat(Est.S_dir,char(dirr{j,1}));
    end
    else
    end
    
    disp('finding possibilities : done')
    
%% Calculating centerlines for all possibilities 
    disp('Calculating centerlines for all possibilities')

    fcl={'','',''};
    fIcl={'','',''};
    for j=1:3
        try
            dd=dirr{j,1};
            I=Est.data;
            es=dd;
            W=Est.cell;
            
            %% centerline_from_mask
            
            % Fill any holes
            Io = I;
            I = imfill(I,'holes');
            sizeIo = size(I);
            
            % Crop the mask so channel intersects image boundaries
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            Icrop=I;
            % Original image size
            sizeIo = size(Icrop);

            % Find x,y coordinates of mask boundaries
            boundaries = cell2mat(bwboundaries(Icrop));
            x = boundaries(:,2);  
            y = boundaries(:,1);  
            
            % Perform cropping; order of cropping is important (SE, then NW)
            if ismember('S',es) == 1
                yBottom = ceil(max(y));
                Icrop(yBottom+1:end,:)=[];
            else
                yBottom = sizeIo(1);
            end
            if ismember('E',es) == 1
                xRight = ceil(max(x));
                Icrop(:,xRight+1:end) = [];
            else
                xRight = sizeIo(2);
            end
            if ismember('N',es) == 1
                yTop = ceil(min(y));
                Icrop(1:yTop-1,:)=[];
            else
                yTop = 1;
            end
            if ismember('W',es) == 1
                xLeft = ceil(min(x));
                Icrop(:,1:xLeft-1) = [];
            else
                xLeft = 1;
            end

            % Fill in any holes in the edges of the binary mask
            % Bottom
            Icrop(end+1,:) = true;
            Icrop = imfill(Icrop,'holes');
            % Top
            Icrop(2:end,:) = Icrop(1:end-1,:);
            Icrop(1,:) = true;
            Icrop = imfill(Icrop,'holes');
            Icrop(1,:) = [];
            % Right
            Icrop(:,end+1) = true;
            Icrop = imfill(Icrop,'holes');
            % Left
            Icrop(:,2:end) = Icrop(:,1:end-1);
            Icrop(:,1) = true;
            Icrop = imfill(Icrop,'holes');
            Icrop(:,1) = [];
            I=Icrop;
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            [nr, nc] = size(I);
            
            % Extend the ends of the mask using mirroring so that the centerline ends
            % can be resolved
            % Create image of original mask combined with mirrored images
            Imirror = rot90(I,2);
            Icombined = false(size(I)*3);
            Icombined(nr:nr*2-1,nc:nc*2-1) = I;
            
            % Fill in the combined image with mirrored pads
            for k = 1:2
                if strcmp(es(k),'N')
                    idx_I = find(Icombined(nr,:)==1,1,'first');
                    idx_Im = find(Imirror(end,:)==1,1,'first');
                    Icombined(1:nr,idx_I-idx_Im:idx_I-idx_Im+nc-1) = Imirror; 
                elseif strcmp(es(k),'E')
                    idx_I = find(Icombined(:,2*nc-1)==1,1,'first');
                    idx_Im = find(Imirror(:,1)==1,1,'first');
                    Icombined(idx_I-idx_Im:idx_I-idx_Im+nr-1,2*nc:3*nc-1) = Imirror; 
                elseif strcmp(es(k),'S')
                    idx_I = find(Icombined(2*nr-1,:)==1,1,'first');
                    idx_Im = find(Imirror(1,:)==1,1,'first');
                    Icombined(2*nr:3*nr-1,idx_I-idx_Im:idx_I-idx_Im+nc-1) = Imirror; 
                elseif strcmp(es(k),'W')
                    idx_I = find(Icombined(:,nc)==1,1,'first');
                    idx_Im = find(Imirror(:,end)==1,1,'first');
                    Icombined(idx_I-idx_Im:idx_I-idx_Im+nr-1,1:nc) = Imirror; 
                end
            end
            % Crop the combined, padded image
            nnn=ceil(max(sizeIo)/min(sizeIo));
            padwidth = floor(max(sizeIo)/nnn);
            
            a1=nr-padwidth; a2=nr*2-1+padwidth; b1=nc-padwidth; b2=nc*2-1+padwidth;
            if a1<1
                a1=1;
            end
            if b1<1
                b1=1;
            end
            if a2>length(Icombined(:,1))
                a2=length(Icombined(:,1));
            end
            if b2>length(Icombined(1,:))
                b2=length(Icombined(1,:));
            end
            I = Icombined(a1:a2,b1:b2);
            
            % Fill any holes again (in case some appeared due to mirroring)
            I = imfill(I,'holes');
            
            % Remove any spurious small patches by keeping only the largest blob
            sp = regionprops(I,'Area','PixelIdxList');
            [~,maxidx] = max([sp(:).Area]);
            I = false(size(I));
            I(sp(maxidx).PixelIdxList) = true;
            
            % Perform skeletonization
            Iskel = bwmorph(I,'skel',Inf);
            Iskel = bwmorph(Iskel,'thin');
            Iskel = bwareaopen(Iskel, 10); % In some cases, single isolated pixels remain in the skeleton. This removes such instances.
            
            % Find endpoints of centerline
            E = bwmorph(Iskel, 'endpoints');
            [yE,xE] = find(E);
            for k = 1:2
                if strcmp(es(k),'N')
                    [~,epidx(k)] = min(yE);
                elseif strcmp(es(k),'E')
                    [~,epidx(k)] = max(xE);
                elseif strcmp(es(k),'S')
                    [~,epidx(k)] = max(yE);
                elseif strcmp(es(k),'W')
                    [~,epidx(k)] = min(xE);
                end
            end
            endpts(1,:) = [yE(epidx(1)),xE(epidx(1))];
            endpts(2,:) = [yE(epidx(2)),xE(epidx(2))];
            
            % Find centerline skeleton
            D1 = bwdistgeodesic(Iskel, endpts(1,2), endpts(1,1));
            D2 = bwdistgeodesic(Iskel, endpts(2,2), endpts(2,1));
            Dadd = D1 + D2;
            Dadd = round(Dadd * 8) / 8;
            Dadd(isnan(Dadd)) = inf;
            Iskel = imregionalmin(Dadd);
            Iskel = bwmorph(Iskel,'thin'); 
            
            % Remove mirrored padding from skeleton image
            Iskel = Iskel(padwidth:nr+padwidth-1,padwidth:nc+padwidth-1);
            % Iskel = bwareaopen(Iskel,W*5);

            % Put skeleton back into uncropped original reference frame
            Icl = false(sizeIo);
            Icl(yTop:yTop+nr-1,xLeft:xLeft+nc-1) = Iskel;
            
            % Find endpoints of binary centerline
            [yend, xend] = find(bwmorph(Icl,'endpoints'));    
            
            %%%%%%%%%%%%%%%%%%%%%%
            % Find upstream->downstream ordered centerline coordinates
            Iskel=Icl;
            xst=xend(end);
            yst=yend(end);
            
            x = xst;
            y = yst;
            c = 1; 
            [nr, nc] = size(Iskel);
            
            while 1
                if y(c)-1 > 0 && Iskel(y(c)-1,x(c)) == 1  % N
                    x(c+1) = x(c);
                    y(c+1) = y(c) - 1;
                    Iskel(y(c),x(c)) = 0;
                elseif x(c) + 1 <= nc && Iskel(y(c),x(c)+1) == 1  % E
                    x(c+1) = x(c) + 1;
                    y(c+1) = y(c);
                    Iskel(y(c),x(c)) = 0;
                elseif y(c) + 1 <= nr && Iskel(y(c)+1,x(c)) == 1  % S
                    x(c+1) = x(c);
                    y(c+1) = y(c) + 1;
                    Iskel(y(c),x(c)) = 0;
                elseif x(c) - 1 > 0 && Iskel(y(c),x(c)-1) == 1  % W
                    x(c+1) = x(c) - 1;
                    y(c+1) = y(c);
                    Iskel(y(c),x(c)) = 0;
                elseif y(c)-1 > 0 && x(c) + 1 <= nc && Iskel(y(c)-1,x(c)+1) == 1  % NE
                    x(c+1) = x(c) + 1;
                    y(c+1) = y(c) - 1;
                    Iskel(y(c),x(c)) = 0;
                elseif y(c) + 1 <= nr && x(c) + 1 <= nc && Iskel(y(c)+1,x(c)+1) == 1  % SE
                    x(c+1) = x(c) + 1;
                    y(c+1) = y(c) + 1;
                    Iskel(y(c),x(c)) = 0;
                elseif y(c) + 1 <= nr && x(c) - 1 > 0 && Iskel(y(c)+1,x(c)-1) == 1  % SW
                    x(c+1) = x(c) - 1;
                    y(c+1) = y(c) + 1;
                    Iskel(y(c),x(c)) = 0;
                elseif y(c)-1 > 0 && x(c) - 1 > 0 && Iskel(y(c)-1,x(c)-1) == 1  % NW
                    x(c+1) = x(c) - 1;
                    y(c+1) = y(c) - 1;
                    Iskel(y(c),x(c)) = 0;
                elseif c > length(Iskel)*3
                    break
                else
                    break
                end
                c = c + 1;
            end 
            xy = [x; y]';
            
            % Order coordinates upstream->downstream
            if es(1) == 'W'
                if xy(1,1) > xy(end,1)
                    xy = flipud(xy);
                end
            elseif es(1) == 'N'
                if xy(1,2) > xy(end,2)
                    xy = flipud(xy);
                end
            elseif es(1) == 'E'
                if xy(1,1) < xy(end,1)
                    xy = flipud(xy);
                end
            elseif es(1) == 'S'
                if xy(1,2) < xy(end,2)
                    xy = flipud(xy);
                end
            end
            cl=xy;
            %%%%%%%%%%%%%%%%%%%
            %%
            fcl{j}=cl; fIcl{j}=Icl;
        catch ME
            continue;
        end
    end
    
    disp('Calculating centerlines for all possibilities : done')
    
%% Picking centerline
    disp('Picking centerline')

    for j=1:3
        cl=fcl{1,j};
        if ~isempty(cl)
            S(j) = max([0; cumsum(sqrt(diff(cl(:,1)).^2+diff(cl(:,2)).^2))]);
        else, S(j)=nan;
        end
    end

    Est.rad=(Est.M_width/2+100)/Est.cell;
    C=zeros(3,1)*nan;
    for j=1:3
        if ~isempty(fcl{1,j})
            if strcmp(dirr{j,1}(2),'N')
                a=find(Est.data(1,:)==1);
                b=a(a-fcl{1,j}(end,1)<Est.rad);
                C(j)=sum(Est.data(1,b));
            elseif strcmp(dirr{j,1}(2),'E')
                a=find(Est.data(:,end)==1);
                b=a(a-fcl{1,j}(end,2)<Est.rad);
                C(j)=sum(Est.data(b,end));
            elseif strcmp(dirr{j,1}(2),'W')
                a=find(Est.data(:,1)==1);
                b=a(a-fcl{1,j}(end,2)<Est.rad);
                C(j)=sum(Est.data(b,1));
            elseif strcmp(dirr{j,1}(2),'S')
                a=find(Est.data(end,:)==1);
                b=a(a-fcl{1,j}(end,1)<Est.rad);
                C(j)=sum(Est.data(end,b));
            end
        else
        end
    end
    s=sort(S);
    if max(diff(s))>round((Est.col/100)*(Est.row/100))
        ind=find(S==max(S),1);
    else
        ind=find(C==max(C),1);
    end
    
    disp('Picking centerline : done')
    
%% Choosing data
    disp('Choosing data')

    if ~isempty(Est.data)
    Est.dir=dirr{ind,1};
    Est.cl=fcl{1,ind};
    Est.Icl=fIcl{1,ind};
    else
    end
    
    disp('Choosing data : done')
    
%% Centerline   
    disp('Centerline')
        
    if ~isempty(Est.data)
    a=(Est.cl(:,1)-Est.x_point).^2+(Est.cl(:,2)-Est.y_point).^2;
    [~,a1]=sort(a); a2=a1(1);
    
    Est.Length=[0; cumsum(sqrt(diff(Est.cl(:,1)).^2+diff(Est.cl(:,2)).^2))]*Est.cell;
    ELength=[0; cumsum(sqrt(diff(Est.cl(1:a2,1)).^2+diff(Est.cl(1:a2,2)).^2))]*Est.cell;
    ELength=max(ELength);
    Est.Length=Est.Length-ELength;
    Est.Rind=Est.Length>=0 & [1:length(Est.Length)]'<=length(Est.Length)-2;
    Est.Length=movmean(Est.Length,5,'omitnan');
    
    clear A B a1 a2 ELength ind ind1
    else
    end
    
    disp('Centerline : done')
    
%% Save

    EST_C(i,1).name=A1(i).name;
    EST_C(i,1).start_x=Est.x_start;
    EST_C(i,1).start_y=Est.y_start;
    EST_C(i,1).start_x_coor=Est.x_start_coor;
    EST_C(i,1).start_y_coor=Est.y_start_coor;
    EST_C(i,1).point_x=Est.x_point;
    EST_C(i,1).point_y=Est.y_point;
    EST_C(i,1).cell=Est.cell;
    EST_C(i,1).M_width=Est.M_width;
    EST_C(i,1).col=Est.col;
    EST_C(i,1).row=Est.row;
    EST_C(i,1).rad=Est.rad;
    EST_C(i,1).S_dir=Est.S_dir;
    EST_C(i,1).dir=Est.dir;
    EST_C(i,1).data=Est.data;
    EST_C(i,1).cl=Est.cl;
    EST_C(i,1).Icl=Est.Icl;
    EST_C(i,1).Rind=Est.Rind;
    EST_C(i,1).Length=Est.Length;
    
    disp({i,'/',n})
    
end
    disp('centerline_from_mask_MorphEst : done')
end