function [EST_W]=width_from_mask_MorphEst(EST_B)

% Modified from RivMAP's width_from_mask function (Schwenk et al.,
% 2016)
%
% Copyright (c) 2017, Jon
% Copyright (c) 2005-prese, Jos van der Geest
% Copyright (c) 2017, Douglas M. Schwarz
% Copyright (c) 2012, John D'Errico
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
% 
% * Redistributions of source code must retain the above copyright notice, this
%   list of conditions and the following disclaimer.
% 
% * Redistributions in binary form must reproduce the above copyright notice,
%   this list of conditions and the following disclaimer in the documentation
%   and/or other materials provided with the distribution
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
% FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
% DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
% CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
% OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
% 
% Computes average channel width for segments of centerline. First,
% segments are computed based ont he input spacing variable. Then for each
% segment, the length is computed. The channel area of each segment is then
% computed using a buffer polygon. The average width of the segment is then
% simply the segment area / segment length.
%
% The modified version now sucessfully measures width for N-S directed
% transects, which creates empty outputs in the previous version.
%
% Requires the Image Processing toolbox (poly2mask command)
%
% INPUTS :
%           EST_B - Matlab file containing output of the
%           bankline_from_mask_MorphEst tool
%
%
% OUTPUTS :
%           EST_W - Matlab file containing information about the average
%           channel width at each segment and downstream distance of middle
%           of each segment

[n,~]=size(EST_B);

for i=1:n
%% Width

    disp('Banks')
    
    %%
    cl=EST_B(i).cl;
    lb=EST_B(i).bank{1};
    rb=EST_B(i).bank{2};
    W=EST_B(i).rad;
    %%%%%%%%%%%%%%%%%%%%%%%%%
    dx = W*3;   % how far away from centerline to search for intersection with bank
    % Initialize storage
    width = nan(length(cl),1);

    % First centerline point width will be NaN.
    ncrop = 100; % number of indices away from cl pt to crop banks for intersection search
    L = length(lb);
    R = length(rb);

    % parfor a = 2:length(cl)-1;
    for a = 2:length(cl)-1

        tpx = cl(a,1);
        tpy = cl(a,2);
        % Slope of line from current centerline point to point on circle
        m = (tpy-cl(a-1,2))/(tpx-cl(a-1,1));
        % Slope of perpendicular line
        minv = -1/m;
        % Construct perpendicular lines
        upper_pt = [tpx + dx, tpy + dx*minv];
        lower_pt = [tpx - dx, tpy - dx*minv];
        perpx = [upper_pt(1) lower_pt(1)];
        perpy = [upper_pt(2) lower_pt(2)];

        % Crop banklines for faster intersections search
        % Find index of bank points closest to cl pt
        [~,lbclosestidx] = min(sqrt((lb(:,1)-tpx).^2+(lb(:,2)-tpy).^2));
        [~,rbclosestidx] = min(sqrt((rb(:,1)-tpx).^2+(rb(:,2)-tpy).^2));

        if ~isfinite(minv)
            width(a) = sqrt((lb(lbclosestidx,2)-rb(rbclosestidx,2))^2+(lb(lbclosestidx,1)-rb(rbclosestidx,1))^2);
        else
            if lbclosestidx > ncrop && rbclosestidx > ncrop && ...
                    ncrop < L-lbclosestidx-1 && ncrop < R-rbclosestidx-1
                lbcrop = lb(lbclosestidx-ncrop+1:lbclosestidx+ncrop-1,:);
                rbcrop = rb(rbclosestidx-ncrop+1:rbclosestidx+ncrop-1,:);
            else
                lbcrop = lb;
                rbcrop = rb;
            end
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            x1=perpx;
            y1=perpy;
            x2=lbcrop(:,1);
            y2=lbcrop(:,2);
            robust=1;
            
            % Force all inputs to be column vectors.
            x1 = x1(:);
            y1 = y1(:);
            x2 = x2(:);
            y2 = y2(:);

            % Compute number of line segments in each curve and some differences we'll
            % need later.
            n1 = length(x1) - 1;
            n2 = length(x2) - 1;
            xy1 = [x1 y1];
            xy2 = [x2 y2];
            dxy1 = diff(xy1);
            dxy2 = diff(xy2);

            % Determine the combinations of i and j where the rectangle enclosing the
            % i'th line segment of curve 1 overlaps with the rectangle enclosing the
            % j'th line segment of curve 2.
            [k,l] = find(repmat(min(x1(1:end-1),x1(2:end)),1,n2) <= ...
                repmat(max(x2(1:end-1),x2(2:end)).',n1,1) & ...
                repmat(max(x1(1:end-1),x1(2:end)),1,n2) >= ...
                repmat(min(x2(1:end-1),x2(2:end)).',n1,1) & ...
                repmat(min(y1(1:end-1),y1(2:end)),1,n2) <= ...
                repmat(max(y2(1:end-1),y2(2:end)).',n1,1) & ...
                repmat(max(y1(1:end-1),y1(2:end)),1,n2) >= ...
                repmat(min(y2(1:end-1),y2(2:end)).',n1,1));

            k = reshape(k,[],1);
            l = reshape(l,[],1);
            
            self_intersect = false;

            if self_intersect
                remove = isnan(sum(dxy1(k,:) + dxy2(l,:),2)) | l <= k + 1;
            else
                remove = isnan(sum(dxy1(k,:) + dxy2(l,:),2));
            end
            k(remove) = [];
            l(remove) = [];

            n = length(k);
            T = zeros(4,n);
            AA = zeros(4,4,n);
            AA([1 2],3,:) = -1;
            AA([3 4],4,:) = -1;
            AA([1 3],1,:) = dxy1(k,:).';
            AA([2 4],2,:) = dxy2(l,:).';
            B = -[x1(k) x2(l) y1(k) y2(l)].';

            if robust
                overlap = false(n,1);
                warning_state = warning('off','MATLAB:singularMatrix');
                % Use try-catch to guarantee original warning state is restored.
                try
                    lastwarn('')
                    for kk = 1:n
                        T(:,kk) = AA(:,:,kk)\B(:,kk);
                        [unused,last_warn] = lastwarn;
                        lastwarn('')
                        if strcmp(last_warn,'MATLAB:singularMatrix')
                            % Force in_range(k) to be false.
                            T(1,kk) = NaN;
                            % Determine if these segments overlap or are just parallel.
                            overlap(kk) = rcond([dxy1(k(kk),:);xy2(l(kk),:) - xy1(k(kk),:)]) < eps;
                        end
                    end
                    warning(warning_state)
                catch err
                    warning(warning_state)
                    rethrow(err)
                end
                % Find where t1 and t2 are between 0 and 1 and return the corresponding
                % x0 and y0 values.
                in_range = (T(1,:) >= 0 & T(2,:) >= 0 & T(1,:) <= 1 & T(2,:) <= 1).';
                % For overlapping segment pairs the algorithm will return an
                % intersection point that is at the center of the overlapping region.
                if any(overlap)
                    ia = k(overlap);
                    ja = l(overlap);
                    % set x0 and y0 to middle of overlapping region.
                    T(3,overlap) = (max(min(x1(ia),x1(ia+1)),min(x2(ja),x2(ja+1))) + ...
                        min(max(x1(ia),x1(ia+1)),max(x2(ja),x2(ja+1)))).'/2;
                    T(4,overlap) = (max(min(y1(ia),y1(ia+1)),min(y2(ja),y2(ja+1))) + ...
                        min(max(y1(ia),y1(ia+1)),max(y2(ja),y2(ja+1)))).'/2;
                    selected = in_range | overlap;
                else
                    selected = in_range;
                end
                xy0 = T(3:4,selected).';

                % Remove duplicate intersection points.
                [xy0,index] = unique(xy0,'rows');
                x0 = xy0(:,1);
                y0 = xy0(:,2);

            else % non-robust option
                for k = 1:n
                    [L,U] = lu(AA(:,:,k));
                    T(:,k) = U\(L\B(:,k));
                end

                % Find where t1 and t2 are between 0 and 1 and return the corresponding
                % x0 and y0 values.
                in_range = (T(1,:) >= 0 & T(2,:) >= 0 & T(1,:) < 1 & T(2,:) < 1).';
                x0 = T(3,in_range).';
                y0 = T(4,in_range).';
            end
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            xl=x0;
            yl=y0;
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            xr=perpx;
            yr=perpy;
            x2=rbcrop(:,1);
            y2=rbcrop(:,2);
            robust=1;
            
            % Force all inputs to be column vectors.
            x1 = x1(:);
            y1 = y1(:);
            x2 = x2(:);
            y2 = y2(:);

            % Compute number of line segments in each curve and some differences we'll
            % need later.
            n1 = length(x1) - 1;
            n2 = length(x2) - 1;
            xy1 = [x1 y1];
            xy2 = [x2 y2];
            dxy1 = diff(xy1);
            dxy2 = diff(xy2);

            % Determine the combinations of i and j where the rectangle enclosing the
            % i'th line segment of curve 1 overlaps with the rectangle enclosing the
            % j'th line segment of curve 2.
            [k,l] = find(repmat(min(x1(1:end-1),x1(2:end)),1,n2) <= ...
                repmat(max(x2(1:end-1),x2(2:end)).',n1,1) & ...
                repmat(max(x1(1:end-1),x1(2:end)),1,n2) >= ...
                repmat(min(x2(1:end-1),x2(2:end)).',n1,1) & ...
                repmat(min(y1(1:end-1),y1(2:end)),1,n2) <= ...
                repmat(max(y2(1:end-1),y2(2:end)).',n1,1) & ...
                repmat(max(y1(1:end-1),y1(2:end)),1,n2) >= ...
                repmat(min(y2(1:end-1),y2(2:end)).',n1,1));

            k = reshape(k,[],1);
            l = reshape(l,[],1);
            
            self_intersect = false;

            if self_intersect
                remove = isnan(sum(dxy1(k,:) + dxy2(l,:),2)) | l <= k + 1;
            else
                remove = isnan(sum(dxy1(k,:) + dxy2(l,:),2));
            end
            k(remove) = [];
            l(remove) = [];

            n = length(k);
            T = zeros(4,n);
            AA = zeros(4,4,n);
            AA([1 2],3,:) = -1;
            AA([3 4],4,:) = -1;
            AA([1 3],1,:) = dxy1(k,:).';
            AA([2 4],2,:) = dxy2(l,:).';
            B = -[x1(k) x2(l) y1(k) y2(l)].';

            if robust
                overlap = false(n,1);
                warning_state = warning('off','MATLAB:singularMatrix');
                % Use try-catch to guarantee original warning state is restored.
                try
                    lastwarn('')
                    for kk = 1:n
                        T(:,kk) = AA(:,:,kk)\B(:,kk);
                        [unused,last_warn] = lastwarn;
                        lastwarn('')
                        if strcmp(last_warn,'MATLAB:singularMatrix')
                            % Force in_range(k) to be false.
                            T(1,kk) = NaN;
                            % Determine if these segments overlap or are just parallel.
                            overlap(kk) = rcond([dxy1(k(kk),:);xy2(l(kk),:) - xy1(k(kk),:)]) < eps;
                        end
                    end
                    warning(warning_state)
                catch err
                    warning(warning_state)
                    rethrow(err)
                end
                % Find where t1 and t2 are between 0 and 1 and return the corresponding
                % x0 and y0 values.
                in_range = (T(1,:) >= 0 & T(2,:) >= 0 & T(1,:) <= 1 & T(2,:) <= 1).';
                % For overlapping segment pairs the algorithm will return an
                % intersection point that is at the center of the overlapping region.
                if any(overlap)
                    ia = k(overlap);
                    ja = l(overlap);
                    % set x0 and y0 to middle of overlapping region.
                    T(3,overlap) = (max(min(x1(ia),x1(ia+1)),min(x2(ja),x2(ja+1))) + ...
                        min(max(x1(ia),x1(ia+1)),max(x2(ja),x2(ja+1)))).'/2;
                    T(4,overlap) = (max(min(y1(ia),y1(ia+1)),min(y2(ja),y2(ja+1))) + ...
                        min(max(y1(ia),y1(ia+1)),max(y2(ja),y2(ja+1)))).'/2;
                    selected = in_range | overlap;
                else
                    selected = in_range;
                end
                xy0 = T(3:4,selected).';

                % Remove duplicate intersection points.
                [xy0,index] = unique(xy0,'rows');
                x0 = xy0(:,1);
                y0 = xy0(:,2);

            else % non-robust option
                for k = 1:n
                    [L,U] = lu(AA(:,:,k));
                    T(:,k) = U\(L\B(:,k));
                end

                % Find where t1 and t2 are between 0 and 1 and return the corresponding
                % x0 and y0 values.
                in_range = (T(1,:) >= 0 & T(2,:) >= 0 & T(1,:) < 1 & T(2,:) < 1).';
                x0 = T(3,in_range).';
                y0 = T(4,in_range).';
            end
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            xr=x0;
            yr=y0;
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            nl = numel(xl); % number of left-bank intersections
            nr = numel(xr); % number of right-bank intersections
            if  nl == 0 || nr == 0   % Skip if lines don't intersect both banks - either increase dx or accept NaN value
            else
                % In case multiple intersections are returned, use the closest
                center_to_lb = sqrt((xl-cl(a,1)).^2+(yl-cl(a,2)).^2);
                center_to_lb = min(center_to_lb);
                center_to_rb = sqrt((xr-cl(a,1)).^2+(yr-cl(a,2)).^2);
                center_to_rb = min(center_to_rb);
                width(a) = center_to_lb + center_to_rb;
            end
        end
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    EST_B(i).Wbl=width;
    EST_B(i).Wbl=EST_B(i).Wbl*EST_B(i).cell;
    
    %%
    
    EST_B(i).Wbl=movmean(EST_B(i).Wbl,5,'omitnan');
    
    disp('Banks :done')
    
%% Save

    EST_W=EST_B;
    
    disp({i,'/',n})    
    
end
    disp('width_from_mask_MorphEst : done')
end